#WinBUGS DIVERSITY MODEL

library(coda)
library(R2WinBUGS)
library(tcltk)

####BEFORE YOU START -CHANGE THESE IF NECESSARY

#directory with data and model files

wd<-"c:/DIVBUGS"
setwd(wd)

#Path to WinBUGS

bd<-"C:/program files/WinBUGS14/"

#Data file (see example for structure)

mydata<-"testbugs.txt"

Niters<-7000
Thin<-3

#####################################################
##hpd function written by Martyn Plummer
hpd <- function(x, coverage=0.95)
{
    require(coda)
    #x <- as.matrix(x)
    out <- matrix(NA, nrow=nvar(x), ncol=3)
    rownames(out) <- varnames(x)
    colnames(out) <- c("mode", "lower", "upper")

        f <- function(p) {
        ## Function to find coverage probability of HPD credible set
        ## at level p.  Estimated coverage is a step function. We
        ## use linear interpolation to smooth it out
        if (p == density.range[2]) {
            set.coverage <- 0
        }
        else {
            p.upper <- min(y.density$y[y.density$y > p])
            p.lower <- max(y.density$y[y.density$y <= p])
           
            cov.upper <- sum(y.counts[y.density$y >= p.upper])/sum(y.counts)
            cov.lower <- sum(y.counts[y.density$y >= p.lower])/sum(y.counts)
           
            c <- (p.upper - p)/(p.upper - p.lower)
            set.coverage <- c * cov.upper + (1 - c) * cov.lower
        }

        return(set.coverage - coverage)
    }

    for (i in 1:nvar(x)) {
        y <- unclass(x[,i])
        y.density <- density(y, n=1024)
        m <- length(y.density$x)
        ## Find the mode
        out[i,1] <- y.density$x[which.max(y.density$y)]
        ## density evaluates the density function on a regular grid
        ## of values. We bin the observations y according to the
        ## closest grid value
        dx <- diff(range(y.density$x))/m
        breaks <- c(y.density$x[1] - dx/2, y.density$x + dx/2)
        y.counts <- hist(y, breaks=breaks, plot=FALSE)$counts
        density.range <- range(y.density$y)
        uniroot.out <- uniroot(f, density.range)
        ## Assuming that we have a single interval, find the limits
        out[i,2:3] <- range(y.density$x[y.density$y > uniroot.out$root])
        ## Check!
        if (sum(abs(diff(y.density$y > uniroot.out$root))) != 2) {
            warning("HPD set is not a closed interval for variable ",
                    varnames(x)[i])
        }
    }
    return(out)
}
######################################################################
Runmodel<-function(whatmodel="LogNormal"){
setwd(wd)
x<-read.table(mydata,header=T)
x<-as.matrix(x)
x<-t(x)
spp<-ncol(x)
plots<-nrow(x)
data <- list ("x", "spp", "plots")


if (whatmodel=="LogNormal"){
model.file<-"lognorm3.odc"
parameters<<-c("Sigm","Shan","Simp","InvS","Hill","PieJ","Equi","DfSi","DfSh","DfIS","DfHi","DfPJ","DfEq")
}

if (whatmodel=="Gamma"){
model.file<-"Gamma3.odc"
parameters<<-c("Alph","Shan","Simp","InvS","Hill","PieJ","Equi","DfSi","DfSh","DfIS","DfHi","DfPJ","DfEq","Shap","Scal")
}

div.sim <- bugs(data=data,parameters.to.save=parameters, model.file=model.file, bugs.directory =bd,inits="NULL",n.chains=1,codaPkg = TRUE,DIC=T,n.iter = Niters,n.burnin=1000,n.thin=Thin,debug=F)
CodOut<<-read.coda(out=paste(div.sim,".out",sep=""),ind=paste(div.sim,".ind",sep=""))
cis<<-hpd(CodOut)
print(cis)
c1<<-as.data.frame(unclass(CodOut))
}

#######################################################
bplot<-function(prm="Sigm"){
boxplot(c1[,grep(prm,names(c1))],boxwex=0.2,col="grey90")
baseline<-median(as.numeric(unlist(c1[,grep(prm,names(c1))])))
basetext<-"Overall median"
if (substr(prm,1,2)=="Df")
{baseline<-0
basetext<-"Baseline (zero)"}
abline(h=baseline,lty=3)
cis<-cis[grep(prm,row.names(cis)),]
 for (i in 1:length(cis[,1])){
  points(i,cis[i,1],col=2,pch="X")
  arrows(i,cis[i,1],i,cis[i,2],angle=90,lty=2)
  arrows(i,cis[i,1],i,cis[i,3],angle=90,lty=2)
}
legend(locator(1),
c("Mode","HPD 95% CI",basetext),
col = c(2,1,1),
lty=c(0,2,3),
pch=c("X"," "," "),
lwd=1,
cex=0.7)
}
#################################################################################

#######################################################
dplot<-function(prm="Sigm"){
plot(CodOut[,grep(prm,names(c1))])
}
#################################################################################


#################################################################################
#A Rudimentary GUI

require(tcltk)
mets<-c("Alph","Sigm","Shan","Simp","InvS","Hill","PieJ","Equi","DfSi","DfSh","DfIS","DfHi","DfPJ","DfEq","Shap","Scal")
firstshot<-TRUE

tt<-tktoplevel()
tkgrid(tklabel(tt,text="Change any of the elements below before running the model."))
###########
Path1 <- tclVar(wd)
entry.Path1 <-tkentry(tt,width="40",textvariable=Path1)
tkgrid(tklabel(tt,text="Path to working directory with data and model files."),entry.Path1)

Onenter <- function()
{
    wd <<- tclvalue(Path1)
    setwd(wd)
    bd <<- tclvalue(Path2)
    mydata <<- tclvalue(my.data)
    Niters <<- as.numeric(tclvalue(n.iters))+1000
    Thin <<- as.numeric(tclvalue(n.thin))

}
tkbind(entry.Path1, "<Return>",Onenter)
##################
Path2 <- tclVar(bd)
entry.Path2 <-tkentry(tt,width="40",textvariable=Path2)
tkgrid(tklabel(tt,text="Path to WinBUGS"),entry.Path2)
tkbind(entry.Path2, "<Return>",Onenter)
###################

my.data <- tclVar(mydata)
entry.data <-tkentry(tt,width="40",textvariable=my.data)
tkgrid(tklabel(tt,text="Data file"),entry.data)
tkbind(entry.data, "<Return>",Onenter)
##############################
n.iters <- tclVar(Niters)
entry.iters <-tkentry(tt,width="40",textvariable=n.iters)
tkgrid(tklabel(tt,text="Number of iterations (after 1000 iteration burn in)."),entry.iters)
tkbind(entry.iters, "<Return>",Onenter)
############################################
n.thin <- tclVar(Thin)
entry.thin <-tkentry(tt,width="5",textvariable=n.thin)
tkgrid(tklabel(tt,text="Thin the chain by a factor of ...."),entry.thin)
tkbind(entry.thin, "<Return>",Onenter)
##################################################
rb1 <- tkradiobutton(tt)
rb2 <- tkradiobutton(tt)
rbValue <- tclVar("Log Normal")
tkconfigure(rb1,variable=rbValue,value="LogNormal")
tkconfigure(rb2,variable=rbValue,value="Gamma")
tkgrid(tklabel(tt,text="Which model to run?"))
tkgrid(tklabel(tt,text="Log Normal "),tklabel(tt,text="Gamma "))
tkgrid(rb1,rb2)

OnRun <- function()
{
    Onenter()
    whatmodel <<- as.character(tclvalue(rbValue))
    Runmodel(whatmodel)
    tkfocus(tt)

}
Run.but <- tkbutton(tt,text="Run model",command=OnRun)
tkgrid(Run.but,columnspan=2)

##########################################
Ondiag1 <- function()print(heidel.diag(c))
Ondiag2 <- function()print(geweke.diag(c))
diag1.but <-tkbutton(tt,text="Run Heidelberger Welch diagnostics",command=Ondiag1)
diag2.but <-tkbutton(tt,text="Run Geweke diagnostic",command=Ondiag2)
tkgrid(diag1.but,columnspan=2)
tkgrid(diag2.but,columnspan=2)
tkgrid(tklabel(tt,text="    ")) # Blank line

#####################################################################################

scr <- tkscrollbar(tt, repeatinterval=5,
                   command=function(...)tkyview(tl,...))
tl<-tklistbox(tt,height=6,selectmode="single",yscrollcommand=function(...)tkset(scr,...),background="white")
#tl<-tklistbox(tt,height=6,selectmode="single",background="white")

tkgrid(tklabel(tt,text="Choose a metric for plotting"),columnspan=2)
tkgrid(tklabel(tt,text="    "))
tkgrid(tl,scr,columnspan=1)
tkgrid.configure(tl,rowspan=6,sticky="e")
tkgrid.configure(scr,rowspan=6,sticky="nsw")

    for (i in (1:length(mets)))
    {
        tkinsert(tl,"end",mets[i])
    }
    tkselection.set(tl,2)
 

####################################3
Onbp <- function()
{  
if(firstshot==TRUE){
ReturnVal <- tkmessageBox(title="Boxplots",message="Click within the plot area to add legend, or right click to cancel the legend",icon="info",type="ok")
firstshot<<-FALSE}

    metChoice <- mets[as.numeric(tkcurselection(tl))+1]
    prm <<- paste(metChoice)
    bplot(prm)}

Onpl <- function()
{    metChoice <- mets[as.numeric(tkcurselection(tl))+1]
    prm <<- paste(metChoice)
    dplot(prm)
}


tkgrid(tklabel(tt,text="    ")) # Blank line

bp.but <-tkbutton(tt,text="Box plots",command=Onbp) 
pl.but <-tkbutton(tt,text="Trace and density plots",command=Onpl)
tkgrid(bp.but,columnspan=2);tkgrid(pl.but,columnspan=2)
tkgrid(tklabel(tt,text="    ")) # Blank line
tkfocus(tt)










